#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import os, string, sys
import math
from array import array

#get pixel helper function
def get_pixel(point,image_width,p_size,src_pixels) :
	x = int(point[0])
	y = int(point[1])
	src_pos = (x + image_width * y) * p_size
	pix = src_pixels[src_pos: src_pos + p_size]
	if p_size == 3:    #if it doesn't have alpha information add full opacity
		pix.append(255)
	return (pix)
def set_pixel(point,image_width,p_size,src_pixels,color):
	x = int(point[0])
	y = int(point[1])
	src_pos = (x + image_width * y) * p_size
	#src_pixels[src_pos: src_pos + p_size] = array("B",color).tostring()
	src_pixels[src_pos] = color[0]
	src_pixels[src_pos+1] = color[1]
	src_pixels[src_pos+2] = color[2]
	src_pixels[src_pos+3] = color[3]
	return None
	
def distance(p1,p2):
	return (((p2[0] - p1[0])**2 + (p2[1] - p1[1])**2)**0.5)
	
def round_border_raster(image,layer,ok):
	pdb.gimp_image_undo_group_start(image)
	
	pdb.gimp_layer_resize_to_image_size(layer) # make layer same as image size to not deal with possible layer offsets
	#pdb.gimp_layer_resize_to_image_size(layer2)
	
	#creates a new layer and name it
	#new_layer = gimp.Layer(image, "Intersect Raster", image.width, image.height, RGBA_IMAGE,
    #                       100, NORMAL_MODE)
	#image.add_layer(new_layer, 0)
	#couldn't call these below properly not sure why. but found above code online on some examples
	#new_layer = pdb.gimp_layer_new(image,image.width,image.height,RGB_IMAGE,"Intersect raster",100,NORMAL_MODE)
	#pdb.gimp_image_insert_layer(image,new_layer,0,0)
	
	#grab layer's pixel information
	srcRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height, True, False)
	p_size = len(srcRgn[0,0])
	
	src_pixels = array("B", srcRgn[0:layer.width, 0:layer.height])
	
	if pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_message("Layer has alpha. OK!")
	else:
		raise Exception("You need to add alpha channel to layer!")
	#selRgn = image.selection.get_pixel_rgn(0, 0, layer.width, layer.height, True, False)
	#pdb.gimp_message(str(image.selection))
	
	selection = pdb.gimp_selection_bounds(image)
	selected = selection[0]
	if selected < 1:
		pdb.gimp_message("You need to make a selection!")
		raise Exception("You need to make a selection!")
		
		
	x1 = selection[1]
	y1 = selection[2]
	x2 = selection[3]
	y2 = selection[4]
	
	
	
	border = []
	inpoints = []
	gimp.progress_init("Finding vertical border pixels...")
	for y in range(y1,y2+1):
		gimp.progress_update(1.0*(y-y1)/(y2+1-y1))
		x = x1
		if pdb.gimp_selection_value(image,x,y) > 0:
			border.append((x,y)) #add point to it
			inpoints.append((x,y))
		x = x2
		if pdb.gimp_selection_value(image,x,y) > 0:
			border.append((x,y)) #add point to it
			inpoints.append((x,y))
	gimp.progress_init("Finding horizontal border pixels...")
	for x in range(x1,x2+1):
		gimp.progress_update(1.0*(x-x1)/(x2+1-x1))
		y = y1
		if pdb.gimp_selection_value(image,x,y) > 0:
			border.append((x,y)) #add point to it
			inpoints.append((x,y))
		y = y2
		if pdb.gimp_selection_value(image,x,y) > 0:
			border.append((x,y)) #add point to it
			inpoints.append((x,y))
	gimp.progress_init("Finding inner border pixels...")
	for y in range(y1+1, y2):
		gimp.progress_update(1.0*(y-y1)/(y2-y1))
		for x in range(x1+1, x2):
			if (pdb.gimp_selection_value(image,x,y) > 0) and ((pdb.gimp_selection_value(image,x-1,y) == 0) or (pdb.gimp_selection_value(image,x-1,y-1) == 0) or (pdb.gimp_selection_value(image,x,y-1) == 0) or (pdb.gimp_selection_value(image,x+1,y-1) == 0) or (pdb.gimp_selection_value(image,x+1,y) == 0) or (pdb.gimp_selection_value(image,x+1,y+1) == 0) or (pdb.gimp_selection_value(image,x,y+1) == 0) or (pdb.gimp_selection_value(image,x-1,y+1) == 0)):
				border.append((x,y))
			if (pdb.gimp_selection_value(image,x,y) > 0):
				inpoints.append((x,y))
	
	

	
	pdb.gimp_message(str(len(border)))
	
	for point in inpoints:
		set_pixel(point,layer.width,p_size,src_pixels,[0,0,255,255])
		
	for pixel in border:
		set_pixel(pixel,layer.width,p_size,src_pixels,[255,0,0,255])
	
	#shortest_distances=[[0]] * len(inpoints)
	to_points = []
	to_distances = []
	for i in range(0,len(border)):
		to_points.append([])
		to_distances.append([0])
	to_border = [0] * len(inpoints)  #points to which border index we're close to
	gimp.progress_init("Calculating pixels to border...")
	for i in range(0,len(inpoints)):
		gimp.progress_update(1.0*i/len(inpoints))
		point1 = inpoints[i]
		min_dist = 1000000
		save_index = -1
		for j in range(0,len(border)):
			point2 = border[j]
			cur_dist = distance(point1,point2)
			if cur_dist <= min_dist:
				min_dist = cur_dist
				save_index = j
		for j in range(0,len(border)):
			point2 = border[j]
			cur_dist = distance(point1,point2)
			if round(cur_dist) == round(min_dist):
				to_points[j].append(point1) #add the point to indicate it belongs to this border
				to_distances[j].append(min_dist) #add the distance into to_distances to calculate max later
		to_border[i] = save_index               #point to the save_index to point to the right border
	radius_ = [0] * len(border)
	gimp.progress_init("Calculating maximum radiuses...")
	for i in range(0,len(border)):
		gimp.progress_update(1.0*i/len(border))
		radius_[i] = max(to_distances[i])
	
	max_radius = max(radius_)
	gimp.progress_init("Rendering...")
	for i in range(0, len(inpoints)):
		gimp.progress_update(1.0*i/len(inpoints))
		dist = distance(inpoints[i],border[to_border[i]])
		base = radius_[to_border[i]] - dist
		height = ((radius_[to_border[i]]**2)-(base**2))**0.5
		c_value = 255 - int((height/max_radius) * 255.0)
		set_pixel(inpoints[i],layer.width,p_size,src_pixels,[c_value, c_value, c_value, 255])
	
	
	srcRgn[0:layer.width, 0:layer.height] = src_pixels.tostring()
	layer.flush()
	layer.update(0,0,layer.width,layer.height)
	
	pdb.gimp_displays_flush()
	pdb.gimp_message("Done!")	
	
	pdb.gimp_image_undo_group_end(image)
	
register(
    "python_fu_round_borders_raster",
    "Creates round blending to nearest border pixels",
    "Creates round blending to nearest border pixels",
    "Tin Tran",
    "GPL",
    "May 2016",
    "<Image>/Python-Fu/Round Border Raster Experiment...",
    "*",
    [
    #(PF_IMAGE, "image", "Input image", None), #current image
    #(PF_DRAWABLE, "layer2", "Intersect with layer", None), #current layer
    #(PF_STRING, "ok","Just OK to Continue", "OK"),
	#(PF_OPTION,"leftright",   "Direction:", 0, ["Left To Right","Right To Left"]),
	(PF_STRING, "ok","Just OK to Continue", "OK"),
	#(PF_DRAWABLE, "layer2", "Intersect with layer", None), #another layer to be selected to intersect with
	],
    [],
    round_border_raster)

main()